# SPDX-FileCopyrightText: 2025 Espressif Systems (Shanghai) CO LTD
# SPDX-License-Identifier: Apache-2.0

import textwrap

from idf_ci.profiles import IniProfileManager, TomlProfileManager


class TestIniProfile:
    def test_read_valid_profile(self, tmp_path):
        profile_path = tmp_path / 'valid_profile.ini'
        profile_path.write_text(
            textwrap.dedent("""
            [section]
            key=value
        """)
        )

        manager = IniProfileManager([profile_path], profile_path)
        profile = manager.read(profile_path)
        assert profile['section']['key'] == 'value'

    def test_read_nonexistent_profile(self, tmp_path):
        profile_path = tmp_path / 'nonexistent_profile.ini'
        manager = IniProfileManager([profile_path], profile_path)
        profile = manager.read(profile_path)
        assert profile == {}

    def test_merge_multiple_profiles(self, tmp_path):
        profile1_path = tmp_path / 'profile1.ini'
        profile2_path = tmp_path / 'profile2.ini'
        profile1_path.write_text(
            textwrap.dedent("""
            [section1]
            key1=value1
        """)
        )
        profile2_path.write_text(
            textwrap.dedent("""
            [section1]
            key2=value2
        """)
        )

        manager = IniProfileManager([profile1_path, profile2_path], profile1_path)
        merged_profile = manager.read(manager.merged_profile_path)
        assert merged_profile['section1']['key1'] == 'value1'
        assert merged_profile['section1']['key2'] == 'value2'

    def test_merge_with_default_profile(self, tmp_path):
        default_profile_path = tmp_path / 'default_profile.ini'
        profile_path = tmp_path / 'profile.ini'
        default_profile_path.write_text(
            textwrap.dedent("""
            [default_section]
            default_key=default_value
        """)
        )
        profile_path.write_text(
            textwrap.dedent("""
            [section]
            key=value
        """)
        )

        manager = IniProfileManager(['default', profile_path], default_profile_path)
        merged_profile = manager.read(manager.merged_profile_path)
        assert merged_profile['default_section']['default_key'] == 'default_value'
        assert merged_profile['section']['key'] == 'value'


class TestTomlProfile:
    def test_read_valid_profile(self, tmp_path):
        profile_path = tmp_path / 'valid_profile.toml'
        profile_path.write_text(
            textwrap.dedent("""
            [section]
            key = 'value'
        """)
        )

        manager = TomlProfileManager([profile_path], profile_path)
        profile = manager.read(profile_path)
        assert profile['section']['key'] == 'value'

    def test_read_nonexistent_profile(self, tmp_path):
        profile_path = tmp_path / 'nonexistent_profile.toml'
        manager = TomlProfileManager([profile_path], profile_path)
        profile = manager.read(profile_path)
        assert profile == {}

    def test_merge_multiple_profiles(self, tmp_path):
        profile1_path = tmp_path / 'profile1.toml'
        profile2_path = tmp_path / 'profile2.toml'
        profile1_path.write_text(
            textwrap.dedent("""
            [section1]
            key1 = 'value1'

            [section1.key3]
            k3 = 'v3'
            k5 = 'v5'
        """)
        )
        profile2_path.write_text(
            textwrap.dedent("""
            non_section_key = 'non_section_value'
            [section1]
            key2 = 'value2'

            [section1.key3]
            k4 = 'v4'
            k5 = 'v55'
        """)
        )

        manager = TomlProfileManager([profile1_path, profile2_path], profile1_path)
        merged_profile = manager.read(manager.merged_profile_path)
        assert merged_profile['section1']['key1'] == 'value1'
        assert merged_profile['section1']['key2'] == 'value2'
        assert merged_profile['section1']['key3'] == {
            'k3': 'v3',
            'k4': 'v4',
            'k5': 'v55',
        }
        assert merged_profile['non_section_key'] == 'non_section_value'

    def test_merge_with_default_profile(self, tmp_path):
        default_profile_path = tmp_path / 'default_profile.toml'
        profile_path = tmp_path / 'profile.toml'
        default_profile_path.write_text(
            textwrap.dedent("""
            [default_section]
            default_key = 'default_value'
        """)
        )
        profile_path.write_text(
            textwrap.dedent("""
            [section]
            key = 'value'
        """)
        )

        manager = TomlProfileManager(['default', profile_path], default_profile_path)
        merged_profile = manager.read(manager.merged_profile_path)
        assert merged_profile['default_section']['default_key'] == 'default_value'
        assert merged_profile['section']['key'] == 'value'



# SPDX-FileCopyrightText: 2025 Espressif Systems (Shanghai) CO LTD
# SPDX-License-Identifier: Apache-2.0

import logging
import os
import typing as t

from _pytest.python import Function
from pytest_embedded.plugin import parse_multi_dut_args

from idf_ci.utils import to_list

LOGGER = logging.getLogger(__name__)


class PytestApp:
    """
    Represents a pytest app.
    """

    def __init__(self, path: str, target: str, config: str) -> None:
        self.path = os.path.abspath(path)
        self.target = target
        self.config = config or 'default'

    def __hash__(self) -> int:
        return hash((self.path, self.target, self.config))

    @property
    def build_dir(self) -> str:
        """
        Returns the build directory for the app.

        .. note::

            Matches the build_dir (by default build_@t_@w) in the build profile.

        :return: The build directory for the app.
        """
        return os.path.join(self.path, f'build_{self.target}_{self.config}')


class PytestCase:
    """
    Represents a pytest test case.
    """

    def __init__(self, apps: t.List[PytestApp], item: Function) -> None:
        self.apps = apps
        self.item = item

    @classmethod
    def get_param(cls, item: Function, key: str, default: t.Any = None) -> t.Any:
        # funcargs is not calculated while collection
        # callspec is something defined in parametrize
        if not hasattr(item, 'callspec'):
            return default

        return item.callspec.params.get(key, default) or default

    @classmethod
    def from_item(cls, item: Function) -> t.Optional['PytestCase']:
        """
        Turn pytest item to PytestCase
        """
        count = cls.get_param(item, 'count', 1)

        # default app_path is where the test script locates
        app_paths = to_list(parse_multi_dut_args(count, cls.get_param(item, 'app_path', os.path.dirname(item.path))))
        configs = to_list(parse_multi_dut_args(count, cls.get_param(item, 'config', 'default')))
        targets = to_list(parse_multi_dut_args(count, cls.get_param(item, 'target', None)))  # defined fixture

        if targets is None or targets == [None] * count:
            LOGGER.critical(
                'No "target" defined in `@pytest.mark.parametrize` for test function "%s" in file "%s. Skipping...',
                item.name,
                item.path,
            )
            return None

        return PytestCase(
            apps=[PytestApp(app_paths[i], targets[i], configs[i]) for i in range(count)],
            item=item,
        )

    def __hash__(self) -> int:
        return hash((self.path, self.name, self.apps, self.all_markers))

    @property
    def path(self) -> str:
        return str(self.item.path)

    @property
    def name(self) -> str:
        return self.item.originalname

    @property
    def targets(self) -> t.List[str]:
        return [app.target for app in self.apps]

    @property
    def configs(self) -> t.List[str]:
        return [app.config for app in self.apps]

    @property
    def caseid(self) -> str:
        if self.is_single_dut:
            return f'{self.targets[0]}.{self.configs[0]}.{self.name}'
        else:
            return f'{tuple(self.targets)}.{tuple(self.configs)}.{self.name}'

    @property
    def is_single_dut(self) -> bool:
        return True if len(self.apps) == 1 else False

    @property
    def is_host_test(self) -> bool:
        return 'host_test' in self.all_markers or 'linux' in self.targets

    @property
    def is_in_ci(self) -> bool:
        return 'CI_JOB_ID' in os.environ or 'GITHUB_ACTIONS' in os.environ

    @property
    def target_selector(self) -> str:
        return ','.join(app.target for app in self.apps)

    @property
    def all_markers(self) -> t.Set[str]:
        return {marker.name for marker in self.item.iter_markers()}

    def get_skip_reason_if_not_built(self, app_dirs: t.Optional[t.List[str]] = None) -> t.Optional[str]:
        """
        Check if all binaries of the test case are built in the app lists.

        :param app_dirs: app folder paths to check
        :return: debug string if not all binaries are built in the app lists, None otherwise
        """
        if app_dirs is None:
            # ignore this feature
            return None

        bin_found = [0] * len(self.apps)
        for i, app in enumerate(self.apps):
            if app.build_dir in app_dirs:
                bin_found[i] = 1

        if sum(bin_found) == 0:
            msg = f'Skip test case {self.name} because all following binaries are not listed in the app lists: '
            for app in self.apps:
                msg += f'\n - {app.build_dir}'

            return msg

        if sum(bin_found) == len(self.apps):
            return None

        # some found, some not, looks suspicious
        msg = f'Found some binaries of test case {self.name} are not listed in the app lists.'
        for i, app in enumerate(self.apps):
            if bin_found[i] == 0:
                msg += f'\n - {app.build_dir}'

        msg += '\nMight be a issue of .build-test-rules.yml files'
        return msg



# SPDX-FileCopyrightText: 2025 Espressif Systems (Shanghai) CO LTD
# SPDX-License-Identifier: Apache-2.0
import functools
import os

import click

from idf_ci._compat import UNDEF


########################
# Validation Functions #
########################
def _semicolon_separated_list(ctx, param, value):  # noqa: ARG001
    if value is None:
        return UNDEF

    if not isinstance(value, str):
        raise click.BadParameter('Value must be a string')

    return [v.strip() for v in value.split(';') if v.strip()]


###########
# Options #
###########
_OPTION_PATHS_HELP = """
List of directories to process. Support passing multiple times.

\b
Example:
    --paths component_1 --paths component_2
    -p component_1 -p component_2
"""


def option_paths(func):
    return click.option(
        '--paths',
        '-p',
        default=[os.getcwd()],
        multiple=True,
        type=click.Path(dir_okay=True, file_okay=False, exists=True),
        help=_OPTION_PATHS_HELP,
    )(func)


def option_target(func):
    return click.option(
        '--target', '-t', default='all', help='Target to be processed. Or "all" to process all targets.'
    )(func)


_OPTION_PROFILES_HELP = """
\b
List of profiles to apply. Could be "default" or file path to a custom profile.
Support passing multiple ones separated by a semicolon (;).
The later profiles will override the previous ones.

\b
Example:
  --profiles default;custom.toml  # To apply default and custom profiles
  --profiles ';'  # To unset the default profile
"""


def option_profiles(func):
    return click.option(
        '--profiles',
        help=_OPTION_PROFILES_HELP,
        callback=_semicolon_separated_list,
    )(func)


def option_parallel(func):
    @click.option('--parallel-count', default=1, help='Number of parallel builds')
    @click.option('--parallel-index', default=1, help='Index of the parallel build')
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        return func(*args, **kwargs)

    return wrapper


def option_modified_files(func):
    return click.option(
        '--modified-files',
        help='Semicolon separated list of files that have been modified',
        callback=_semicolon_separated_list,
    )(func)



# SPDX-FileCopyrightText: 2025 Espressif Systems (Shanghai) CO LTD
# SPDX-License-Identifier: Apache-2.0

import os
import re

from idf_ci.cli import cli
from idf_ci.settings import CiSettings


def test_default_component_mapping_regexes():
    expected_regexes = [
        '/components/(.+)/',
        '/common_components/(.+)/',
    ]
    assert CiSettings().component_mapping_regexes == expected_regexes


def test_default_component_ignored_file_extensions():
    expected_extensions = [
        '.md',
        '.rst',
        '.yaml',
        '.yml',
        '.py',
    ]
    assert CiSettings().component_ignored_file_extensions == expected_extensions


def test_get_modified_components():
    test_files = [
        'components/wifi/wifi.c',
        'components/bt/bt_main.c',
        'common_components/esp_common/test.c',
        'docs/example.md',  # should be ignored
        'random/file.txt',  # should not match any component
    ]

    expected_components = {'wifi', 'bt', 'esp_common'}
    assert CiSettings().get_modified_components(test_files) == expected_components


def test_ignored_file_extensions():
    test_files = [
        'components/wifi/README.md',
        'components/bt/docs.rst',
        'components/esp_common/config.yaml',
        'components/test/test.yml',
        'components/utils/util.py',
    ]

    assert CiSettings().get_modified_components(test_files) == set()


def test_extended_component_mapping_regexes():
    settings = CiSettings(
        extend_component_mapping_regexes=[
            '/custom/path/(.+)/',
        ]
    )

    test_files = [
        'custom/path/my_component/test.c',
        'components/wifi/wifi.c',
    ]

    expected_components = {'my_component', 'wifi'}
    assert settings.get_modified_components(test_files) == expected_components


def test_extended_ignored_extensions():
    settings = CiSettings(
        extend_component_ignored_file_extensions=[
            '.txt',
            '.json',
        ]
    )

    test_files = [
        'components/wifi/test.txt',
        'components/bt/config.json',
        'components/esp_common/main.c',
    ]

    expected_components = {'esp_common'}
    assert settings.get_modified_components(test_files) == expected_components


def test_all_component_mapping_regexes():
    patterns = CiSettings().all_component_mapping_regexes
    assert len(patterns) == 2

    test_path = '/components/test_component/test.c'
    for pattern in patterns:
        match = pattern.search(test_path)
        if '/components/(.+)/' in pattern.pattern:
            assert match is not None
            assert match.group(1) == 'test_component'


def test_component_mapping_with_absolute_paths():
    abs_path = os.path.abspath('components/wifi/wifi.c')
    components = CiSettings().get_modified_components([abs_path])
    assert components == {'wifi'}


def test_ci_profile_option(tmp_dir, runner):
    # Create a custom CI profile file
    custom_profile = os.path.join(tmp_dir, 'custom_ci_profile.toml')
    with open(custom_profile, 'w') as f:
        f.write("""
extend_component_mapping_regexes = [
    '/custom/path/(.+)/'
]

component_ignored_file_extensions = [
    '.custom'
]
""")

    result = runner.invoke(cli, ['--ci-profile', custom_profile, 'build', 'run', '--paths', tmp_dir])
    assert result.exit_code == 0
    assert f'Using CI profile: {custom_profile}' in result.output
    assert CiSettings.CONFIG_FILE_PATH == custom_profile
    assert len(CiSettings().all_component_mapping_regexes) == 3  # default got 2

    CiSettings.CONFIG_FILE_PATH = None  # reset

    # Test with non-existent profile
    non_existent = os.path.join(tmp_dir, 'non_existent.toml')
    result = runner.invoke(cli, ['--ci-profile', non_existent])
    assert result.exit_code == 2  # Click returns 2 for parameter validation errors
    assert re.search(r"Error: Invalid value for '--ci-profile': File .* does not exist.", result.output)
    assert len(CiSettings().all_component_mapping_regexes) == 2  # default got 2


def test_ci_profile_not_specified(runner):
    # Test default behavior when no profile is specified
    original_config_path = CiSettings.CONFIG_FILE_PATH
    result = runner.invoke(cli, ['build', 'init-profile'])
    assert result.exit_code == 0
    assert CiSettings.CONFIG_FILE_PATH == original_config_path



# SPDX-FileCopyrightText: 2025 Espressif Systems (Shanghai) CO LTD
# SPDX-License-Identifier: Apache-2.0

import os
import shutil

import click

from idf_ci._compat import Undefined
from idf_ci.scripts import build as build_cmd
from idf_ci.settings import CiSettings

from ._options import option_modified_files, option_parallel, option_paths, option_profiles, option_target


@click.group()
def build():
    """
    Group of build related commands
    """
    pass


@build.command()
@option_paths
@option_target
@option_profiles
@option_parallel
@option_modified_files
@click.option('--only-test-related', is_flag=True, help='Run build only for test-related apps')
@click.option('--only-non-test-related', is_flag=True, help='Run build only for non-test-related apps')
@click.option('--dry-run', is_flag=True, help='Run build in dry-run mode')
def run(
    *,
    paths,
    target,
    profiles,
    parallel_count,
    parallel_index,
    modified_files,
    only_test_related,
    only_non_test_related,
    dry_run,
):
    """
    Run build according to the given profiles
    """
    if isinstance(profiles, Undefined):
        profiles = CiSettings().build_profiles

    if isinstance(modified_files, Undefined):
        modified_files = None

    click.echo(f'Building {target} with profiles {profiles} at {paths}')
    build_cmd(
        paths,
        target,
        profiles=profiles,
        parallel_count=parallel_count,
        parallel_index=parallel_index,
        modified_files=modified_files,
        only_test_related=only_test_related,
        only_non_test_related=only_non_test_related,
        dry_run=dry_run,
    )


@build.command()
@click.option('--path', default=os.getcwd(), help='Path to create the build profile')
def init_profile(path: str):
    """
    Create .idf_build_apps.toml with default values at the given folder
    """
    if os.path.isdir(path):
        # here don't use idf_build_apps.constants.IDF_BUILD_APPS_TOML_FN
        # since idf_build_apps requires idf_path
        # fix it after idf-build-apps support lazy-load variables
        filepath = os.path.join(path, '.idf_build_apps.toml')
    else:
        filepath = path

    shutil.copyfile(os.path.join(os.path.dirname(__file__), '..', 'templates', 'default_build_profile.toml'), filepath)
    click.echo(f'Created build profile at {filepath}')



# SPDX-FileCopyrightText: 2025 Espressif Systems (Shanghai) CO LTD
# SPDX-License-Identifier: Apache-2.0
import logging
import typing as t

import rich.logging

_T = t.TypeVar('_T')


@t.overload
def to_list(s: None) -> None: ...


@t.overload
def to_list(s: t.Iterable[_T]) -> t.List[_T]: ...


@t.overload
def to_list(s: _T) -> t.List[_T]: ...


def to_list(s):
    """
    Turn all objects to lists

    :param s: anything
    :return:
        - ``None``, if ``s`` is None
        - itself, if ``s`` is a list
        - ``list(s)``, if ``s`` is a tuple or a set
        - ``[s]``, if ``s`` is other type

    """
    if s is None:
        return s

    if isinstance(s, list):
        return s

    if isinstance(s, set) or isinstance(s, tuple):
        return list(s)

    return [s]


def setup_logging(level: t.Optional[int] = logging.WARNING) -> None:
    """
    Setup logging

    :param level: logging level
    """
    package_logger = logging.getLogger(__package__)

    if level is None:
        level = logging.WARNING

    package_logger.setLevel(level)
    package_logger.addHandler(
        rich.logging.RichHandler(
            level=level,
            show_path=False,
            log_time_format='[%Y-%m-%d %H:%M:%S]',
            tracebacks_word_wrap=False,
        )
    )
    package_logger.propagate = False



# SPDX-FileCopyrightText: 2025 Espressif Systems (Shanghai) CO LTD
# SPDX-License-Identifier: Apache-2.0

import os
from pathlib import Path

from idf_ci.cli import cli


def test_build_profile_init(runner, tmp_dir):
    # Test init command with default path
    with runner.isolated_filesystem():
        result = runner.invoke(cli, ['build', 'init-profile', '--path', tmp_dir])
        assert result.exit_code == 0
        assert f'Created build profile at {os.path.join(tmp_dir, ".idf_build_apps.toml")}' in result.output
        assert os.path.exists(os.path.join(tmp_dir, '.idf_build_apps.toml'))

    # Test init command with specific file path
    specific_path = os.path.join(tmp_dir, 'custom_build.toml')
    result = runner.invoke(cli, ['build', 'init-profile', '--path', specific_path])
    assert result.exit_code == 0
    assert f'Created build profile at {specific_path}' in result.output
    assert os.path.exists(specific_path)


def test_ci_profile_init(runner, tmp_dir):
    # Test init command with default path
    with runner.isolated_filesystem():
        result = runner.invoke(cli, ['init-profile', '--path', tmp_dir])
        assert result.exit_code == 0
        assert f'Created CI profile at {os.path.join(tmp_dir, ".idf_ci.toml")}' in result.output
        assert os.path.exists(os.path.join(tmp_dir, '.idf_ci.toml'))

    # Test init command with specific file path
    specific_path = os.path.join(tmp_dir, 'custom_ci.toml')
    result = runner.invoke(cli, ['init-profile', '--path', specific_path])
    assert result.exit_code == 0
    assert f'Created CI profile at {specific_path}' in result.output
    assert os.path.exists(specific_path)


def test_test_profile_init(runner, tmp_dir):
    # Test init command with default path
    with runner.isolated_filesystem():
        result = runner.invoke(cli, ['test', 'init-profile', '--path', tmp_dir])
        assert result.exit_code == 0
        assert f'Created test profile at {os.path.join(tmp_dir, "pytest.ini")}' in result.output
        assert os.path.exists(os.path.join(tmp_dir, 'pytest.ini'))

    # Test init command with specific file path
    specific_path = os.path.join(tmp_dir, 'custom_test.toml')
    result = runner.invoke(cli, ['test', 'init-profile', '--path', specific_path])
    assert result.exit_code == 0
    assert f'Created test profile at {specific_path}' in result.output
    assert os.path.exists(specific_path)


def test_completions(runner):
    result = runner.invoke(cli, ['completions'])
    assert result.exit_code == 0
    assert 'To enable autocomplete run the following command:' in result.output
    assert 'Bash:' in result.output
    assert 'Zsh:' in result.output
    assert 'Fish:' in result.output


def test_profile_init_file_exists(runner, tmp_dir):
    # Test that init doesn't fail if file already exists
    build_profile_path = os.path.join(tmp_dir, '.idf_build_apps.toml')
    ci_profile_path = os.path.join(tmp_dir, '.idf_ci.toml')

    # Create files first
    Path(build_profile_path).touch()
    Path(ci_profile_path).touch()

    # Try to init again
    result = runner.invoke(cli, ['build', 'init-profile', '--path', tmp_dir])
    assert result.exit_code == 0

    result = runner.invoke(cli, ['init-profile', '--path', tmp_dir])
    assert result.exit_code == 0

    result = runner.invoke(cli, ['test', 'init-profile', '--path', tmp_dir])
    assert result.exit_code == 0



# SPDX-FileCopyrightText: 2025 Espressif Systems (Shanghai) CO LTD
# SPDX-License-Identifier: Apache-2.0

from .idf_pytest.models import PytestApp, PytestCase
from .idf_pytest.plugin import IdfPytestPlugin
from .idf_pytest.scripts import get_pytest_cases
from .profiles import IniProfileManager, TomlProfileManager
from .scripts import build, get_all_apps
from .settings import CiSettings

__all__ = [
    'CiSettings',
    'IdfPytestPlugin',
    'IniProfileManager',
    'PytestApp',
    'PytestCase',
    'TomlProfileManager',
    'build',
    'get_all_apps',
    'get_pytest_cases',
]



# SPDX-FileCopyrightText: 2025 Espressif Systems (Shanghai) CO LTD
# SPDX-License-Identifier: Apache-2.0

import configparser
import os.path
import tempfile
import typing as t
from abc import abstractmethod
from collections import defaultdict
from contextlib import contextmanager

from tomlkit import dump, load

from ._compat import UNDEF, PathLike, Undefined
from .settings import CiSettings


def _merge_dicts(source: t.Dict, target: t.Dict) -> t.Dict:
    """Recursively merge two dictionaries."""
    for key, value in target.items():
        if key in source and isinstance(source[key], dict) and isinstance(value, dict):
            _merge_dicts(source[key], value)
        else:
            source[key] = value
    return source


class ProfileManager:
    suffix: t.ClassVar[str] = '.toml'

    def __init__(self, profiles: t.List[PathLike], default_profile_path: PathLike):
        self.profiles = profiles
        self.default_profile_path = default_profile_path

        self._merged_profile_path: str = None  # type: ignore

    @property
    def merged_profile_path(self) -> str:
        if not self._merged_profile_path:
            self.merge()
        return self._merged_profile_path

    def _resolve_profile_path(self, profile: PathLike) -> PathLike:
        """Resolve 'default' to actual path."""
        return self.default_profile_path if profile == 'default' else os.path.expandvars(profile)

    @abstractmethod
    def read(self, profile: PathLike) -> t.Dict:
        """Read a profile file and return the profile as a dictionary."""

    @abstractmethod
    def merge(self) -> None:
        """Merge profiles and write to temporary file and set it as self.merged_profile_path."""

    @contextmanager
    def _merged_profile_writer(self) -> t.Generator[t.IO[str], None, None]:
        with tempfile.NamedTemporaryFile(suffix=self.suffix, mode='w', delete=False) as fw:
            yield fw

            self._merged_profile_path = fw.name


class IniProfileManager(ProfileManager):
    suffix: t.ClassVar[str] = '.ini'

    def read(self, profile: PathLike) -> t.Dict:
        config = configparser.ConfigParser(interpolation=None)

        config.read(profile)

        return {s: dict(config.items(s)) for s in config.sections()}

    def merge(self) -> None:
        merged_dict: t.Dict[str, t.Dict] = defaultdict(dict)

        for profile in self.profiles:
            profile = self._resolve_profile_path(profile)

            # ini file must have a section, so we can directly update the merged_dict
            for section, options in self.read(profile).items():
                merged_dict[section].update(options)

        with self._merged_profile_writer() as fw:
            config = configparser.ConfigParser(interpolation=None)
            for section, options in merged_dict.items():
                config[section] = options
            config.write(fw)


class TomlProfileManager(ProfileManager):
    def read(self, profile: PathLike) -> t.Dict:
        if not os.path.isfile(profile):
            return {}

        with open(profile) as f:
            return load(f)

    def merge(self) -> None:
        merged_dict: t.Dict = {}

        for profile in self.profiles:
            profile = self._resolve_profile_path(profile)

            # toml dict can be nested, so we need to merge it recursively
            merged_dict = _merge_dicts(merged_dict, self.read(profile))

        with self._merged_profile_writer() as fw:
            dump(merged_dict, fw)


def get_build_profile(profiles: t.List[PathLike] = UNDEF) -> TomlProfileManager:  # type: ignore
    if isinstance(profiles, Undefined):
        profiles = CiSettings().build_profiles

    return TomlProfileManager(
        profiles=profiles,
        default_profile_path=os.path.join(os.path.dirname(__file__), 'templates', 'default_build_profile.toml'),
    )


def get_test_profile(profiles: t.List[PathLike] = UNDEF) -> IniProfileManager:  # type: ignore
    if isinstance(profiles, Undefined):
        profiles = CiSettings().test_profiles

    return IniProfileManager(
        profiles=profiles,
        default_profile_path=os.path.join(os.path.dirname(__file__), 'templates', 'default_test_profile.ini'),
    )



# SPDX-FileCopyrightText: 2025 Espressif Systems (Shanghai) CO LTD
# SPDX-License-Identifier: Apache-2.0

import os
import shutil

import click


@click.group()
def test():
    """
    Group of test related commands
    """
    pass


@test.command()
@click.option('--path', default=os.getcwd(), help='Path to create the CI profile')
def init_profile(path: str):
    """
    Create pytest.ini with default values at the given folder
    """
    if os.path.isdir(path):
        filepath = os.path.join(path, 'pytest.ini')
    else:
        filepath = path

    shutil.copyfile(os.path.join(os.path.dirname(__file__), '..', 'templates', 'default_test_profile.ini'), filepath)
    click.echo(f'Created test profile at {filepath}')
